.loading {display: none;}
.fetching .loading {display: block;}

a {
  color: blue;
}

h1 {
  background-color: black;
  margin: 0;
  padding: .25em;
  color: white;
  border-bottom: 5px solid gray;
}

.box {
  border: 2px solid gray;
}

.tweeter {
  margin-right: 360px;
}

ul {
  list-style: none;
  margin: 0;
  padding: 0;
}

li {
  margin: .25em;
  padding: 2px;
}

li img {
  float: left;
  margin: 2px;
  margin-right: .5em;
  max-height: 48px;
  min-height: 48px;
}

li.even {
  background-color: lightgray;
}


.addressbook {
  float: right;
  width: 350px;
}

.addressbook li {
  font-size: .9em;
}

.clrleft {
  clear: left;
}

.notes {
  font-size: .8em;
  color: gray;
}

.username, .nickname {
  font-weight: bold;
}

.editor {
  padding: 4px;
}

label {
  color: gray;
  display: inline-block;
  width: 75px;
  text-align: right;
  padding: 2px;
  margin-top: 10px;
}

.editor input[type=text],
.editor textarea {
  width: 230px;
  vertical-align: text-top;
}
  
.editor TEXTAREA {
  height: 50px;
}

.debug{
  font-size: .7em;
  white-space: pre;
  padding: 0;
  margin: 0;
}